 /****************************************************************************************
 *
 * FILE NAME:       efmcrc.c
 *
 * DESCRIPTION:     Linux EFM module CRC calculation function.
 *
 * (C) Copyright 2007, Texas Instruments Inc
 *
 * Note: This file implement functions to calculate CRC functions for
 *  - (US) 802.3 FCS. (32 bit frame checksum)
 *  - (US) TC-CRC16 (16 bit)
 *  - (DS) TC-CRC16 table (16 bit), load for PDSP.
 *
 * History:
 *   2/20/2007 CPH  CQ11466, Written, add EFM support
 *
 ****************************************************************************************/


#include "efmcrc.h"


#ifdef EFMCRC_STANDALONE
#include <stdio.h>
#include <stdlib.h>
#endif


/* The DS CRC16 table for pdsp */
#include "pdsp_crc16_lut.h" 

static unsigned long update_crc32(unsigned long crc, unsigned char *buf, int len);
static void efmcrc_init_crc32(void);
static void efmcrc_init_crc16(void);


/* CRC32 Table */
#ifdef EFMCRC_USE_INTERNAL_SDRAM
unsigned long  *crc32_table = (unsigned long *) CRC32_START_ADDR;  // last 1K of 4K internal SDRAM.
#ifdef ETH_TC_CRC16
unsigned short *crc16_table = (unsigned short *) CRC16_START_ADDR;
#endif
#else
static unsigned long  crc32_table[CRCTBL_ENTRIES]; // CRC32 table in external SDRAM
#ifdef ETH_TC_CRC16
static unsigned short crc16_table[CRCTBL_ENTRIES]; // CRC16 table in external SDRAM
#endif
#endif

void efmcrc_init_crcTable(void)
{ 
  efmcrc_init_crc32();
#ifdef ETH_TC_CRC16
  efmcrc_init_crc16();
#endif
}


void efmcrc_load_pdsp_crc16_lut(void)
{
  unsigned int address=0xA3008850;  // start of PDSP CRC16 LUT
  int i, j, k=0;

  for (i=0; i<4; i++){
    for (j=0; j<32; j++, address+=4)
    {
      *(unsigned int *)address = pdsp_crc16[k++];
    }
    address = address & 0xffffff00;
    address += 0x150;
  }
}

// Generate CRC32 Table
static void efmcrc_init_crc32(void)
{
  unsigned long c;
  int n, k;

  for (n = 0; n < CRCTBL_ENTRIES; n++) {
    c = (unsigned long) n;
    for (k = 0; k < 8; k++) {
      if (c & 1)
        c = CRC32_POLYNOMIAL ^ (c >> 1);
      else
        c = c >> 1;
    }
    crc32_table[n] = c;
  }
}


/* Update a running CRC with the bytes buf[0..len-1]--the CRC
   should be initialized to all 1's, and the transmitted value
   is the 1's complement of the final running CRC (see the
   crc() routine below).
*/
static unsigned long update_crc32(unsigned long crc, unsigned char *buf, int len)
{
  unsigned long c = crc;
  unsigned char *endptr;

  endptr = buf + len;
  while (endptr != buf)
  {
    c = crc32_table[(c ^ *buf++) & 0xff] ^ (c >> 8);
  }
  return c;
}

/* Return the CRC of the byte buffer buf[0..len-1] */
unsigned long efm_calc_crc32(unsigned char *buf, int len)
{
  return update_crc32(0xffffffffL, buf, len) ^ 0xffffffffL;
}

#ifdef ETH_TC_CRC16

// Generate CRC16 table
static void efmcrc_init_crc16(void)
{
  unsigned short c;
  int n, k;

  for (n = 0; n < CRCTBL_ENTRIES; n++) {
    c = (unsigned short) n;
    for (k = 0; k < 8; k++) {
      if (c & 1)
        c = CRC16_POLYNOMIAL ^ (c >> 1);
      else
        c = c >> 1;
    }
    crc16_table[n] = c;
  }
}


static unsigned short update_crc16(unsigned short crc, unsigned char *buf, int len)
{
  unsigned short c = crc;
  unsigned char *endptr;

  endptr = buf + len;
  while (endptr != buf)
  {
    c = crc16_table[(c ^ *buf++) & 0xff] ^ (c >> 8);
//    c= crc16_table[(c & 0xff)^*buf++] ^ ((c>>8) & 0xffff);
  }
  return c;
}


unsigned short efm_calc_crc16(unsigned char *buf, int len)
{
  return update_crc16(0xffff, buf, len) ^ 0xffff;
}

#endif // ETH_TC_CRC16


#ifdef EFMCRC_STANDALONE

// Stand alone program for verification
int main (int argc, char *argv[])
{
  int i;
  ETH_CRC32 crc;

  efmcrc_init_crc32();
  for (i=0; i<CRCTBL_ENTRIES; i++)
  {
    if ((i%8)==0) printf("\n");
    printf("%08x ", crc32_table[i]);
  }
  crc.x =  0x12345678;
  printf("crc.b[0]=%02x, b[1]=%02x, b[2]=%02x, b[3]=%02x\n",
    crc.b[0], crc.b[1], crc.b[2], crc.b[3]);


  printf("\n\n");
  efmcrc_init_crc16();
  for(i=0; i<CRCTBL_ENTRIES; i++)
  {
    if ((i%8)==0) printf("\n");
    printf("%04x ", crc16_table[i]);
  }

  return 0;
}

#endif
